#line 1 "dmx_fixture.pde"
/************************************************************
* 5-channel RGB DMX fixture controller                      *
*   DMX Controller with RGB+master intensity control plus   *
*   built-in programs.  EEPROM storage of DMX channel and   *
*   operating state.  Button + potentiometer controls and   *
*   4-digit 7-segment LED display.                          *
*                                                           *
* Developed by Dan Julio (www.danjuliodesigns.com)          *
* Rev 1.0 Dec 22, 2010                                      *
* Rev 1.1 Jan 13, 2011 - fixed bug with ScaleColor, other   *
*                        clean up, slightly tighten random  *
*                                                           *
* DMX receiver based on DMX-512 Reception, Developed by Max *
* Pierson                                                   *
*   Version Rev15 9 Oct 2010                                *
*                                                           *
* This software released under the same license as Max's:   *
*   "Released under the WTFPL license, although I would     *
*   appreciate Attribution and Share-Alike                  *
*   (blog.wingedvictorydesign.com)"                         *
*    www.danjuliodesigns.com                                *
*                                                           *
* Modified to decouple processing of data with ISR handling *
* of DMX frames.  The ISRs now run by themselves to detect  *
* each frame.  Remember to replace the HardwareSerial.cpp   *
* file in the Arduino release with the one Max provides.    *
*                                                           *
* This version compiled under Arduino 0016 although it      *
* should compile under later versions.                      *
*                                                           *
* User selectable modes (using push button)                 *
*  DMX: DMX control mode                                    *
*  DMX Addr Set: Set starting DMX address                   *
*    Short button press advances to next digit              *
*    Long button press exits DMX Addr Set mode to DMX mode  *
*    Pot sets value for blinking digit                      *
*  P1: Manual set color (pot sets color)                    *
*  P2: Strobe (using manually set P1 color, pot sets speed) *
*  P3: Sequential fade (pot sets speed)                     *
*  P4: Random fade (pot sets speed)                         *
*                                                           *
* Pressing the button quickly advances between DMX and P1-  *
* P4.  Pressing the button longer than two seconds in DMX   *
* mode causes the fixture to enter DMX Addr Set mode.       *           
*                                                           *
* DMX Channels in DMX mode                                  *
*  0: Mode                                                  *
*       0 -  63 : DMX Color control                         *
*      64 - 127 : Strobe using DMX Color control (value     *
*                 sets flashes/sec)                         *
*     128 - 191 : Sequential fade (value sets speed)        *
*     192 - 255 : Random fade (value sets speed)            *
*  1: Master Intensity                                      *
*  2: Red Channel                                           *
*  3: Green Channel                                         *
*  4: Blue Channel                                          *
*                                                           *
* EEPROM Storage                                            *
*   0: Magic Byte (set to 0xA5 when EEPROM initialized)     *
*   1: DMX Address High [8]                                 *
*   2: DMX Addres Low [7:0]                                 *
*   3: User-set Mode (values 0, 2-5 for UIMode)             *
*   4: P1 Color High [9:8] (used if stored mode is strobe)  *
*   5: P1 Color Low [7:0]                                   *
*                                                           *
* Arduino Port Connections:                                 *
*  D0: RX - TTL RX input from SN75176 driver                *
*  D1: TX - TTL TX output to SN75176 driver (unused)        *
*  D2: nRE - Active low Receiver Enable to SN75176          *
*  D3: DE - Active high Driver Enabled to SN75176           *
*  D4: LED Display 'a' cathode                              *
*  D5: Red PWM Output                                       *
*  D6: Green PWM Output                                     *
*  D7: LED Display 'b' cathode                              *
*  D8: LED Display 'c' cathode                              *
*  D9: Blue PWM Output                                      *
* D10: LED Display 'd' cathode                              *
* D11: LED Display 'e' cathode                              *
* D12: LED Display 'f' cathode                              *
* D13: LED Display 'g' cathode                              *
* A0(D14): LED Display digit 0 anode                        *
* A1(D15): LED Display digit 1 anode                        *
* A2(D16): LED Display digit 2 anode                        *
* A3(D17): LED Display digit 3 anode                        *
* A4(D18): Active low button input                          *
* A5: Potentiometer input (0 - VCC)                         *
*                                                           *
* Notes:                                                    *
*   1. DMX Addressing is limited to a maximum of 506 to     *
*      allow all 5 channels to inhabit legal addresses.     *
*      In practice the DMX receive code timing gets wonnky  *
*      at high addresses and DMX reception may not work.    *
*   2. TX capability is provided in hardware but unused by  *
*       this code (provides for future RDM capability).     *
*                                                           *
*************************************************************/
#include <EEPROM.h>


/******************************* Constant declarations *****************************/

#define NUMBER_OF_CHANNELS 5
#define MAX_DMX_ADDR 506

#define RECEIVER_OUTPUT_ENABLE 2
/* receiver output enable (pin2) on the SN75176.  
*  will be left low to set the SN75176 to receive data. */

#define DRIVER_OUTPUT_ENABLE 3
/* driver output enable (pin3) on the SN75176.  
*  will left low to disable driver output. */

#define RX_PIN 0   // serial receive pin, which takes the incoming data from the .
#define TX_PIN 1   // serial transmission pin

#define PWM_RED 5
#define PWM_GREEN 6
#define PWM_BLUE 9

#define LED_SEG_A 4
#define LED_SEG_B 7
#define LED_SEG_C 8
#define LED_SEG_D 10
#define LED_SEG_E 11
#define LED_SEG_F 12
#define LED_SEG_G 13
#define LED_DIG_0 14  // Repurposing A0-A3 as digital outputs
#define LED_DIG_1 15
#define LED_DIG_2 16
#define LED_DIG_3 17

#define MODE_BUTTON 18     // Repurposing A4 as digital input
#define AN_POT 5           // Analog input

#define DEBOUNCE_CNT 5     // number of evaluation intervals in a debounce period
#define BUTTON_LONG 80     // Long button press threshold in debounce periods

#define DIGIT_BLINK_PRD 12 // Number of complete multiplex periods in a full blink period

#define UI_MODE_DMX 0
#define UI_MODE_SET_ADDR 1
#define UI_MODE_PROG_1 2
#define UI_MODE_PROG_2 3
#define UI_MODE_PROG_3 4
#define UI_MODE_PROG_4 5

#define OP_MODE_DMX 0
#define OP_MODE_SET_ADDR 1
#define OP_MODE_PROG_1 2
#define OP_MODE_PROG_2 3
#define OP_MODE_PROG_3 4
#define OP_MODE_PROG_4 5

#define EEP_MAGIC_ADDR 0
#define EEP_DMX_H_ADDR 1
#define EEP_DMX_L_ADDR 2
#define EEP_USER_MODE  3
#define EEP_C_H_ADDR   4
#define EEP_C_L_ADDR   5
#define EEP_MAGIC_BYTE 0xA5 // The programmer leaves 0xFF as the default value

#define MODE_CHNG_CNT 200   // Number of evaluation cycles in one mode before updating EEPROM


/******************************* DMX ISR variable declarations ********************************/

#include "Arduino.h"
void setup();
void loop();
void EvalTimeBase();
void EvalControls();
unsigned int CalcAvgPotVal();
byte GetPotDigit();
void EvalDisplay();
byte ComputeSegData(unsigned int DigitVal);
void SetSegments (byte SegData);
void SetupDmxDisplay();
void SetupSetAddrDisplay(byte d100, byte d10, byte d1);
void SetBlinkDigit(byte digit);
void SetAddrDigitDisplay(byte digit, byte val);
void SetupProg1Display();
void SetupProg2Display();
void SetupProg3Display();
void SetupProg4Display();
void SetupSetAddr();
void EvalOpSetAddr();
void EvalUiSetAddr();
void EvalDMX();
byte ScaleColor(byte C);
void SetupDMX();
void EvalUiDMX();
void InitController();
void InitVariables();
void EvalLogic ();
void SetPWM(byte r, byte g, byte b);
void SetupProg1();
void EvalOpProg1();
void EvalUiProg1();
void CalcIndexedRGB(unsigned int index);
void SetupProg2();
byte GetStrobeDelay();
void EvalOpProg2();
void EvalUiProg2();
void SetupProg3();
byte GetFadeDelay();
void EvalOpProg3();
void EvalUiProg3();
void SetupProg4();
void SelectRandomColor();
void SetupInterpolator();
byte RoundFrac(long FracNum);
void EvalOpProg4();
void EvalUiProg4();
void CheckEEPsanity();
unsigned int GetDmxAddress();
void StoreDmxAddress(unsigned int workingAddr);
byte GetFixtureMode();
void StoreFixtureMode(byte nMode);
unsigned int GetFixtureIndexedColor();
void StoreFixtureIndexedColor(unsigned int cIndex);
void EvalModeChange();
#line 167
volatile byte i = 0;                         //dummy variable for dmxvalue[]
volatile byte dmxreceived = 0;               //the latest received value
volatile unsigned int dmxcurrent = 0;        //counter variable that is incremented every time we receive a value.
volatile byte dmxvalue[NUMBER_OF_CHANNELS];  //stores the DMX values we're interested in using.
volatile boolean dmxnewvalue = false;            //set to 1 when new dmx values are received.

volatile byte zerocounter = 0;          
/* a counter to hold the number of zeros received in sequence on the serial receive pin.  
*  When we've received a minimum of 11 zeros in a row, we must be in a break.  As written,
*  the timer2 ISR actually checks for 20 zeros in a row, to ensure detecting an 88uS break. */

/*******************************  variable declarations **********************************/

unsigned int dmxaddress;       // The starting dmx address we will be listening to

byte CurRed;
byte CurGreen;
byte CurBlue;

byte NextRed;
byte NextGreen;
byte NextBlue;

unsigned long DeltaVal[3];     // Absolute delta between current and next color
unsigned long IncVal[3];       // Format {integer part[7:0] "." Fraction[15:0]}; range is 0x000000 - 0x010000 (0.0 - 1.0)
unsigned long InterpColor[3];  // Format {0x00, 0x00, Color[7:0] "." Fraction[15:0]}
boolean InterpSign[3];         // 1 = negative, 0 = positive
byte Major;                    // Major axis pointer: 0 = Red, 1 = Green, 2 = Blue

unsigned long PrevTime;
boolean Tick_5ms;

byte DebounceTimer=DEBOUNCE_CNT;
byte ButtonTimer=0;
boolean ButtonPrev = 0;
boolean ButtonReleased = 0;
boolean ButtonDown = 0;
boolean ButtonPressShort = 0;
boolean ButtonPressLong = 0;
unsigned int PotAvgBuff[4];
byte AvgBuffPtr = 0;

byte CurDigit = 0;    // Current digit to update (0-3)
byte DigitVals[4];
byte BlinkDigit;      // 0 = nothing blinking; 1-4 = blink digit 0-3
byte BlinkCount;

byte OPMode;
byte UIMode;
byte PrevUIMode;      // Used to detect mode changes to update EEPROM
byte ModeChangeTimer; // Used to time mode changes so we don't write to EEPROM unnecessarily

byte SetAddrDigit = 0;
byte Addr1s;
byte Addr10s;
byte Addr100s;
byte PrevPotDigit;   // 0 - 9 from the potentiometer setting

unsigned int PrevPotVal;  // Indexed color value for P1

byte ProgDelay;    // Number of evaluation intervals between program actions
unsigned int ProgIndex;


void setup() {
 CheckEEPsanity();
 InitController();
 InitVariables();
}


void loop()  { 
  if (dmxnewvalue) {
    EvalDMX();
    dmxnewvalue = false;
  }
  
  EvalTimeBase();
  if (Tick_5ms) {
    EvalControls();
    EvalDisplay();
    EvalLogic();
  }
}


void EvalTimeBase() {
  unsigned long CurTime = micros();
  unsigned long deltaT;
  
  if (CurTime < PrevTime) {
    // Clock rollover
    deltaT = PrevTime - CurTime;
  } else {
    deltaT = CurTime - PrevTime;
  }
  
  if (deltaT >= 5000) {
    Tick_5ms = true;
    PrevTime = CurTime;
  } else {
    Tick_5ms = false;
  }
}



// Timer2 compare match interrupt vector handler
ISR(TIMER2_COMPA_vect) {
  if (bitRead(PIND, PIND0)) {      // if a one is detected, we're not in a break, reset zerocounter.
    zerocounter = 0;
    }
  else {
    zerocounter++;                 // increment zerocounter if a zero is received.
    if (zerocounter == 20)         // if 20 0's are received in a row (80uS break)
      {   
      bitClear(TIMSK2, OCIE2A);    // disable this interrupt and enable reception interrupt now that we're in a break.
      bitSet(UCSR0B, RXCIE0);
      }
  }
} // end Timer2 ISR



ISR(USART_RX_vect){
  dmxreceived = UDR0;
  /* The receive buffer (UDR0) must be read during the reception ISR, or the ISR will just 
   *  execute again immediately upon exiting. */
 
  // Look for our data window
  if(++dmxcurrent > dmxaddress) {      // check if the current address is the one we want.
    dmxvalue[i++] = dmxreceived;
    if(i == NUMBER_OF_CHANNELS) {
      dmxnewvalue = true;              // set newvalue, so that the action loop can be executed.
      dmxcurrent = 0;                  // reset for the next frame
      zerocounter = 0;
      i = 0;
      bitClear(UCSR0B, RXCIE0);        // Disable receiving serial data
      bitSet(TIMSK2, OCIE2A);          // Enable Timer/Counter2 Output Compare Match A Interrupt for next break
    } 
  }
} // end ISR
#line 1 "controls.pde"
// Constant values for digit selection using the potentiometer
//   1 band for each digit POT_M
//   1 dead band between each digit POT_N
//   10M + 9N = 1024 counts
#define POT_M 75
#define POT_N 30

void EvalControls() {
   ButtonPressShort = false;
   ButtonPressLong = false;
    
  if (--DebounceTimer == 0) {
    DebounceTimer = DEBOUNCE_CNT;
  
    // Sample the pushbutton
    boolean curButton;
    if (digitalRead(MODE_BUTTON) == 1) {
      curButton = false;
    } else {
      curButton = true;
    }
    ButtonReleased = !curButton && ButtonPrev && ButtonDown;
    ButtonDown = curButton && ButtonPrev;
    ButtonPrev = curButton;
    
    if (ButtonReleased) {
      if (ButtonTimer < BUTTON_LONG) {
        // Short button press registered when button released prior to threshold
        ButtonPressShort = true;
      }
    }
    
    if (ButtonDown) {
      if (ButtonTimer == BUTTON_LONG) {
        // Long button press registered as we cross the threshold with the button pressed
        ButtonPressLong = true;
        ButtonTimer++; // Push timer past threshold to prevent it from counting anymore
      } else {
        if (ButtonTimer < BUTTON_LONG) {
          ButtonTimer++;
        }
      }
    } else {
      ButtonTimer = 0;
    }

    // Sample and update our average potentiometer array
    unsigned int Temp;
    Temp = analogRead(AN_POT);
    PotAvgBuff[AvgBuffPtr] = Temp;
    AvgBuffPtr = (AvgBuffPtr + 1) & 0x03;
  }
}


unsigned int CalcAvgPotVal() {
  unsigned int RetVal = 0;
  
  for (byte i=0; i<4; i++) {
    RetVal += PotAvgBuff[i];
  }
  return (RetVal >> 2);
}


byte GetPotDigit() {
  unsigned int CurPotVal;
  byte i;
  
  CurPotVal = CalcAvgPotVal();
  for (i=0; i<10; i++) {
    if (CurPotVal < ((i+1)*POT_N + (i*POT_M))) {
      return i;
    }
  }
  return 9;  // special case for few counts above final computed
             // value due to rounding errors in M and N
}
#line 1 "display.pde"
// Segment bit values {g, f, e, d, c, b, a}
#define DISPLAY_0 0x3F
#define DISPLAY_1 0x06
#define DISPLAY_2 0x5B
#define DISPLAY_3 0x4F
#define DISPLAY_4 0x66
#define DISPLAY_5 0x6D
#define DISPLAY_6 0x7D
#define DISPLAY_7 0x07
#define DISPLAY_8 0x7F
#define DISPLAY_9 0x6F
#define DISPLAY_C 0x39
#define DISPLAY_A 0x77
#define DISPLAY_P 0x73
#define DISPLAY_BLANK 0x00


int SegPins[] = {LED_SEG_A, LED_SEG_B, LED_SEG_C, LED_SEG_D, LED_SEG_E, LED_SEG_F, LED_SEG_G};

void EvalDisplay() {
  byte CurDigitVal;
  
  if (BlinkDigit == (CurDigit + 1)) {
    if (BlinkCount >= (DIGIT_BLINK_PRD / 2)) {
      CurDigitVal = DISPLAY_BLANK;
    } else {
      CurDigitVal = DigitVals[CurDigit];
    }
    if (BlinkCount == DIGIT_BLINK_PRD) {
      BlinkCount = 0;
    } else {
      BlinkCount++;
    }
  } else {
    CurDigitVal = DigitVals[CurDigit];
  }
  
  switch (CurDigit) {
    case 0: 
      digitalWrite(LED_DIG_3, 0);
      SetSegments(CurDigitVal);
      digitalWrite(LED_DIG_0, 1);
      break;
    case 1:
      digitalWrite(LED_DIG_0, 0);
      SetSegments(CurDigitVal);
      digitalWrite(LED_DIG_1, 1);
      break;
    case 2:
      digitalWrite(LED_DIG_1, 0);
      SetSegments(CurDigitVal);
      digitalWrite(LED_DIG_2, 1);
      break;
    case 3:
      digitalWrite(LED_DIG_2, 0);
      SetSegments(CurDigitVal);
      digitalWrite(LED_DIG_3, 1);
  }
  
  if (CurDigit == 3) {
    CurDigit = 0;
  } else {
    CurDigit++;
  }
}


byte ComputeSegData(unsigned int DigitVal) {
  switch (DigitVal) {
    case 0:
      return DISPLAY_0;
    case 1:
      return DISPLAY_1;
    case 2:
      return DISPLAY_2;
    case 3:
      return DISPLAY_3;
    case 4:
      return DISPLAY_4;
    case 5:
      return DISPLAY_5;
    case 6:
      return DISPLAY_6;
    case 7:
      return DISPLAY_7;
    case 8:
      return DISPLAY_8;
    case 9:
      return DISPLAY_9;
    default:
      return DISPLAY_BLANK;
  }
}


void SetSegments (byte SegData) {
  boolean SegVal;
  for (int i=0; i<8; i++) {
    SegVal = ((~SegData & 0x01) == 0x01);
    SegData = SegData >> 1;
    digitalWrite(SegPins[i], SegVal);
  }
}


void SetupDmxDisplay() {
  unsigned int TrueAddress = dmxaddress - 3;
  DigitVals[0] = ComputeSegData(TrueAddress % 10);
  DigitVals[1] = ComputeSegData((TrueAddress / 10) % 10);
  DigitVals[2] = ComputeSegData(TrueAddress / 100);
  DigitVals[3] = DISPLAY_C;
  BlinkDigit = 0;
}


void SetupSetAddrDisplay(byte d100, byte d10, byte d1) {
  DigitVals[0] = ComputeSegData(d1);
  DigitVals[1] = ComputeSegData(d10);
  DigitVals[2] = ComputeSegData(d100);
  DigitVals[3] = DISPLAY_A;
  BlinkDigit = 3;
  BlinkCount = 0;
}


void SetBlinkDigit(byte digit) {
  // digit 0-2 corresponds with BlinkDigit 1-3
  BlinkDigit = digit+1;
  BlinkCount = 0;
}


void SetAddrDigitDisplay(byte digit, byte val) {
  DigitVals[digit] = ComputeSegData(val);
}


void SetupProg1Display() {
  DigitVals[0] = ComputeSegData(1);
  DigitVals[1] = DISPLAY_BLANK;
  DigitVals[2] = DISPLAY_BLANK;
  DigitVals[3] = DISPLAY_P;
  BlinkDigit = 0;
}


void SetupProg2Display() {
  DigitVals[0] = ComputeSegData(2);
  DigitVals[1] = DISPLAY_BLANK;
  DigitVals[2] = DISPLAY_BLANK;
  DigitVals[3] = DISPLAY_P;
  BlinkDigit = 0;
}


void SetupProg3Display() {
  DigitVals[0] = ComputeSegData(3);
  DigitVals[1] = DISPLAY_BLANK;
  DigitVals[2] = DISPLAY_BLANK;
  DigitVals[3] = DISPLAY_P;
  BlinkDigit = 0;
}


void SetupProg4Display() {
  DigitVals[0] = ComputeSegData(4);
  DigitVals[1] = DISPLAY_BLANK;
  DigitVals[2] = DISPLAY_BLANK;
  DigitVals[3] = DISPLAY_P;
  BlinkDigit = 0;
}
#line 1 "dmx_addressing.pde"
void SetupSetAddr() {
  unsigned int TrueAddress = dmxaddress - 3;
  
  OPMode = OP_MODE_SET_ADDR;
  UIMode = UI_MODE_SET_ADDR;
  SetAddrDigit = 2;  // Start with 100s
  SetBlinkDigit(SetAddrDigit);
  Addr1s = TrueAddress % 10;
  Addr10s = (TrueAddress / 10) % 10;
  Addr100s = TrueAddress / 100;
  SetupSetAddrDisplay(Addr100s, Addr10s, Addr1s);
  
  // Get the current potentiometer "digit" value for detecting changes for newly selected digits
  PrevPotDigit = GetPotDigit();
}


void EvalOpSetAddr() {
  // Evaluate digit value change from pot input
  byte CurPotDigit = GetPotDigit();
  if (CurPotDigit != PrevPotDigit) {
    // Update display and working address
    switch (SetAddrDigit) {
      case 0:
        Addr1s = CurPotDigit;
        break;
      case 1:
        Addr10s = CurPotDigit;
        break;
      case 2:
        Addr100s = CurPotDigit;
    }
    SetAddrDigitDisplay(SetAddrDigit, CurPotDigit);
    PrevPotDigit = CurPotDigit;
  }
}


void EvalUiSetAddr() {
  // Look for button presses
  if (ButtonPressShort) {
    // Select the next digit
    if (SetAddrDigit == 0) {
      SetAddrDigit = 2;
    } else {
      SetAddrDigit--;
    }
    SetBlinkDigit(SetAddrDigit);
  } else if (ButtonPressLong) {
    // Store our new address (ensure it's legal)
    dmxaddress = ((Addr100s * 100) + (Addr10s * 10) + Addr1s);
    if (dmxaddress == 0) {
      dmxaddress = 1;
    } else if (dmxaddress > MAX_DMX_ADDR) {
      dmxaddress = MAX_DMX_ADDR;
    }
    dmxaddress += 3;  // account for the dmx receiver
    StoreDmxAddress(dmxaddress);
    
    // Move to DMX mode
    SetupDMX();
    UIMode = UI_MODE_DMX;
    SetupDmxDisplay();
  }
}
#line 1 "dmx_logic.pde"
void EvalDMX() { 
  if (UIMode == UI_MODE_DMX) {
    if (dmxvalue[0] < 64) {
      // Direct DMX color control
      if (OPMode != OP_MODE_DMX) {
        SetupDMX();
      }
    } else if (dmxvalue[0] < 128) {
      // Color strobe effect
      if (OPMode != OP_MODE_PROG_2) {
        SetupProg2();
      }
    } else if (dmxvalue[0] < 192) {
      // Sequential fader
      if (OPMode != OP_MODE_PROG_3) {
        SetupProg3();
      }
    } else {
      // Random fader
      if (OPMode != OP_MODE_PROG_4) {
        SetupProg4();
      }
    }
    
    // Set the color values for modes that require it
    if ((OPMode == OP_MODE_DMX) || (OPMode == OP_MODE_PROG_2)) {
      CurRed = ScaleColor(dmxvalue[2]);
      CurGreen = ScaleColor(dmxvalue[3]);
      CurBlue = ScaleColor(dmxvalue[4]);
      
      // Immediately update the PWM outputs if we are in Direct DMX Control mode
      if (OPMode == OP_MODE_DMX) {
        SetPWM(CurRed, CurGreen, CurBlue);
      }
    }
  }

  return;
}


// Scale the passed color with the master intensity channel
byte ScaleColor(byte C) {
  unsigned int TempInt = C * dmxvalue[1]; // Result {[7:0] "." [7:0]}
  byte newC = (TempInt >> 8);
  
  // Always round the fractional part of the multiplication up to the next int value
  if ((TempInt & 0xFF) > 0x0) {
    // Round up
    return (newC + 1);
  } else {
    // No rounding
    return (newC);
  }
}



void SetupDMX() {
  OPMode = OP_MODE_DMX;
}


void EvalUiDMX() {
  // Look for button presses
  if (ButtonPressShort) {
    // Move to program 1
    UIMode = UI_MODE_PROG_1;
    SetupProg1();
    SetupProg1Display();
  }
  else if (ButtonPressLong) {
    // Move to address setup
    SetupSetAddr();
  }
}
#line 1 "initialization.pde"
void InitController() {
  /************************* RS485 IF configuration ******************************/
  pinMode(RECEIVER_OUTPUT_ENABLE, OUTPUT);
  pinMode(DRIVER_OUTPUT_ENABLE, OUTPUT);
  digitalWrite(RECEIVER_OUTPUT_ENABLE, LOW);
  digitalWrite(DRIVER_OUTPUT_ENABLE, LOW);    //sets pins 3 and 4 to low to enable reciever mode on the SN75176.

  pinMode(RX_PIN, INPUT);  //sets serial pin to receive data

  /******************************* PWM Outputs *********************************/
  pinMode(PWM_RED, OUTPUT);
  analogWrite(PWM_RED, 0);
  pinMode(PWM_GREEN,OUTPUT);
  analogWrite(PWM_GREEN, 0);
  pinMode(PWM_BLUE, OUTPUT);
  analogWrite(PWM_BLUE, 0);
  
  /******************************* LED Outputs *********************************/
  pinMode(LED_SEG_A, OUTPUT);  // Segment outputs are driven to 0 to enable the segment
  digitalWrite(LED_SEG_A, 1);
  pinMode(LED_SEG_B, OUTPUT);
  digitalWrite(LED_SEG_B, 1);
  pinMode(LED_SEG_C, OUTPUT);
  digitalWrite(LED_SEG_C, 1);
  pinMode(LED_SEG_D, OUTPUT);
  digitalWrite(LED_SEG_D, 1);
  pinMode(LED_SEG_E, OUTPUT);
  digitalWrite(LED_SEG_E, 1);
  pinMode(LED_SEG_F, OUTPUT);
  digitalWrite(LED_SEG_F, 1);
  pinMode(LED_SEG_G, OUTPUT);
  digitalWrite(LED_SEG_G, 1);
  
  pinMode(LED_DIG_0, OUTPUT);  // Digit outputs are driven to 1 to enable the digit
  digitalWrite(LED_DIG_0, 0);
  pinMode(LED_DIG_1, OUTPUT);
  digitalWrite(LED_DIG_1, 0);
  pinMode(LED_DIG_2, OUTPUT);
  digitalWrite(LED_DIG_2, 0);
  pinMode(LED_DIG_3, OUTPUT);
  digitalWrite(LED_DIG_3, 0);
  
  /***************************** Control Inputs **********************************/
  pinMode(MODE_BUTTON, INPUT);
  
  /**************************** USART configuration ******************************/
  
  Serial.begin(250000);
  /* Each bit is 4uS long, hence 250Kbps baud rate */
  
  cli(); //disable interrupts while we're setting bits in registers
  
  bitClear(UCSR0B, RXCIE0);  //disable USART reception interrupt
  
  /******************************* Timer2 configuration ***********************************/
  
  //NOTE:  this will disable PWM on pins 3 and 11.
  bitClear(TCCR2A, COM2A1);
  bitClear(TCCR2A, COM2A0); //disable compare match output A mode
  bitClear(TCCR2A, COM2B1);
  bitClear(TCCR2A, COM2B0); //disable compare match output B mode
  bitSet(TCCR2A, WGM21);
  bitClear(TCCR2A, WGM20);  //set mode 2, CTC.  TOP will be set by OCRA.
  
  bitClear(TCCR2B, FOC2A);
  bitClear(TCCR2B, FOC2B);  //disable Force Output Compare A and B.
  bitClear(TCCR2B, WGM22);  //set mode 2, CTC.  TOP will be set by OCRA.
  bitClear(TCCR2B, CS22);
  bitClear(TCCR2B, CS21);
  bitSet(TCCR2B, CS20);     //no prescaler means the clock will increment every 62.5ns (assuming 16Mhz clock speed).
  
  OCR2A = 64;                
  /* Set output compare register to 64, so that the Output Compare Interrupt will fire
  *  every 4uS.  */
  
  bitClear(TIMSK2, OCIE2B);  //Disable Timer/Counter2 Output Compare Match B Interrupt
  bitSet(TIMSK2, OCIE2A);    //Enable Timer/Counter2 Output Compare Match A Interrupt
  bitClear(TIMSK2, TOIE2);   //Disable Timer/Counter2 Overflow Interrupt Enable           
      
  sei();                     //reenable interrupts
 
}


void InitVariables() {
  unsigned int IndexColor;
  
  // Get the DMX address from storage
  
  dmxaddress = GetDmxAddress() + 3;
  /*  this will allow the USART receive interrupt to fire an additional 3 times for every dmx frame.  
  *   Here's why:
  *   Once to account for the fact that DMX addresses run from 0-511, whereas channel numbers
  *        start numbering at 1.
  *   Once for the Mark After Break (MAB), which will be detected by the USART as a valid character 
  *        (a zero, eight more zeros, followed by a one)
  *   Once for the START code that precedes the 512 DMX values (used for RDM).  */
  
  // Get the initial operating mode
  UIMode = GetFixtureMode();
  switch (UIMode) {
    case UI_MODE_PROG_1:
      SetupProg1();
      SetupProg1Display();
      break;
      
    case UI_MODE_PROG_2:
      SetupProg2();
      SetupProg2Display();
      // Then set the initial color from storage
      IndexColor = GetFixtureIndexedColor();
      CalcIndexedRGB(IndexColor);
      break;
      
    case UI_MODE_PROG_3:
      SetupProg3();
      SetupProg3Display();
      break;
      
    case UI_MODE_PROG_4:
      SetupProg4();
      SetupProg4Display();
      break;
      
    default:
      SetupDMX();
      SetupDmxDisplay();
  }
   
  PrevTime = micros();
  PrevUIMode = UIMode;
  ModeChangeTimer = 0;
}
#line 1 "main_logic.pde"
void EvalLogic () {
  // Evaluate the operating logic
  switch (OPMode) {
    // case OP_MODE_DMX is evaluated directly in the main loop for performance reasons
      
    case OP_MODE_SET_ADDR:
      EvalOpSetAddr();
      break;
      
    case OP_MODE_PROG_1:
      EvalOpProg1();
      break;
      
    case OP_MODE_PROG_2:
      EvalOpProg2();
      break;

    case OP_MODE_PROG_3:
      EvalOpProg3();
      break;

    case OP_MODE_PROG_4:
      EvalOpProg4();
  }
  
  // Evaluate the user interface logic  
  switch (UIMode) {
    case UI_MODE_DMX:
      EvalUiDMX();
      break;
      
    case UI_MODE_SET_ADDR:
      EvalUiSetAddr();
      break;

    case UI_MODE_PROG_1:
      EvalUiProg1();
      break;

    case UI_MODE_PROG_2:
      EvalUiProg2();
      break;

    case UI_MODE_PROG_3:
      EvalUiProg3();
      break;

    case UI_MODE_PROG_4:
      EvalUiProg4();
      break;
  }
  
  // Evaluate the mode change logic for mode storage in EEPROM
  EvalModeChange();
}


void SetPWM(byte r, byte g, byte b) {
//  cli();
  analogWrite(PWM_RED, r);
  analogWrite(PWM_GREEN, g);
  analogWrite(PWM_BLUE, b);
//  sei();
}
#line 1 "prog_logic.pde"
void SetupProg1() {
  OPMode = OP_MODE_PROG_1;
  
  // Set a bogus previous pot value to force the first evaluation to set the current color
  PrevPotVal = 1024;
}


void EvalOpProg1() {
  // Prog1 uses the potentiometer to face through a rainbow of color:
  // Black->White->Red->Yellow->Green->Cyan->Blue->Magenta->Red
  //
  // Check for a change in the pot value
  unsigned int CurPotVal;
  CurPotVal = CalcAvgPotVal();
  if (CurPotVal != PrevPotVal) {
    CalcIndexedRGB(CurPotVal);
    SetPWM(CurRed, CurGreen, CurBlue);
    PrevPotVal = CurPotVal;
  }
}


void EvalUiProg1() {
  // Look for button presses (we ignore long button presses in this mode)
  if (ButtonPressShort) {
    // Move to program 2
    UIMode = UI_MODE_PROG_2;
    SetupProg2();
    SetupProg2Display();
  }
}


void CalcIndexedRGB(unsigned int index) {
 // Calculate current color values based on octant
 // Index Values      Color               Actions
 // ----------------------------------------------------
 //    0 -  127       Black -> White      RGB   0 -> 255
 //  128 -  255       White -> Red        GB  255 ->   0
 //  256 -  383       Red -> Yellow       G     0 -> 255
 //  384 -  511       Yellow -> Green     R   255 ->   0
 //  512 -  639       Green -> Cyan       B     0 -> 255
 //  640 -  767       Cyan -> Blue        G   255 ->   0
 //  768 -  895       Blue -> Magenta     R     0 -> 255
 //  896 - 1023       Magenta -> Red      B   255 ->   0
 //
 byte IndexMsb = (index >> 7) & 0x7;  // IndexMsb = index[9:7]
 byte IndexLsb = (index & 0x7F) << 1; // IndexLsb = index[6:0] << 1
 switch (IndexMsb) {
    case 0: //    0 -  127       Black -> White      RGB   0 -> 255
      CurRed = IndexLsb;
      CurGreen = IndexLsb;
      CurBlue = IndexLsb;
      break;
        
    case 1: //  128 -  255       White -> Red        GB  255 ->   0
      CurRed = 255;
      CurGreen = 254 - IndexLsb;
      CurBlue = CurGreen;
      break;
        
    case 2: //  256 -  383       Red -> Yellow       G     0 -> 255
      CurRed = 255;
      CurGreen = IndexLsb;
      CurBlue = 0;
      break;
        
    case 3: //  384 -  511       Yellow -> Green     R   255 ->   0
      CurRed = 254 - IndexLsb;
      CurGreen = 255;
      CurBlue = 0;
      break;
        
    case 4: //  512 -  639       Green -> Cyan       B     0 -> 255
      CurRed = 0;
      CurGreen = 255;
      CurBlue = IndexLsb;
      break;
        
    case 5: //  640 -  767       Cyan -> Blue        G   255 ->   0
      CurRed = 0;
      CurGreen = 254 - IndexLsb;
      CurBlue = 255;
      break;
        
    case 6: //  768 -  895       Blue -> Magenta     R     0 -> 255
      CurRed = IndexLsb;
      CurGreen = 0;
      CurBlue = 255;
      break;
        
    case 7: //  896 - 1023       Magenta -> Red      B   255 ->   0
      CurRed = 255;
      CurGreen = 0;
      CurBlue = 254 - IndexLsb;
  }
}


void SetupProg2() {
  OPMode = OP_MODE_PROG_2;
  
  // Turn off the LEDs
  SetPWM(0, 0, 0);
  
  // Get the initial strobe speed
  ProgDelay = GetStrobeDelay();
}


byte GetStrobeDelay() {
  byte fps;
  
  if (UIMode == UI_MODE_DMX) {
    // Calculate the rate based on channel 0 offset from start of strobe range: 1-33 flashes/sec
    fps = ((dmxvalue[0] - 64) >> 1) + 1;  // Offset/2 + 1
  } else { // UI_MODE_PROG_2
    // Calculate the rate based on the potentiometer value: 1-33 flashes/sec
    fps = (CalcAvgPotVal() >> 5) + 1;     // Avg Pot Val/32 + 1
  }
  return (1000 / fps / 5);
}


void EvalOpProg2() {
  unsigned long PrevFlashTime;
  unsigned long CurFlashTime;
  unsigned long deltaT = 0;
  
  // Prog2 uses the potentiometer or channel 0 offset to select a strobe speed using
  // the CurRed/Green/Blue values.
  //
  // Note: Flash time should be at least one complete PWM cycle (2040 uSec)
  if (--ProgDelay == 0) {
    // Get the current timer value
    PrevFlashTime = micros();
    CurFlashTime = PrevFlashTime;

    // Start the strobe
    SetPWM(CurRed, CurGreen, CurBlue);
    
    // Wait until the strobe period expires
    while (deltaT < 2040) {
      CurFlashTime = micros();
      if (CurFlashTime < PrevFlashTime) {
        deltaT = PrevFlashTime - CurFlashTime;
      } else {
        deltaT = CurFlashTime - PrevFlashTime;
      }
    }
    
    // End the strobe
    SetPWM(0, 0, 0);
    ProgDelay = GetStrobeDelay();
  }
}


void EvalUiProg2() {
  // Look for button presses (we ignore long button presses in this mode)
  if (ButtonPressShort) {
    // Move to program 3
    UIMode = UI_MODE_PROG_3;
    SetupProg3();
    SetupProg3Display();
  }
}


void SetupProg3() {
  OPMode = OP_MODE_PROG_3;
  ProgIndex = 256; // Start with red
  ProgDelay = GetFadeDelay();
}


byte GetFadeDelay() {
  byte nEvalIntervals;
  
  if (UIMode == UI_MODE_DMX) {
    // Calculate the change delay based on channel 0 offset from start of range
    // We guard-band the offset
    //   0-1 => 20 eval intervals per change (10 changes/sec)
    //   2-61 => 20-1 eval intervals per change
    //   62-63 => 1 eval intervals per change (200 changes/sec)
    //
    if (OPMode == OP_MODE_PROG_3) {
      nEvalIntervals = dmxvalue[0] - 128;
    } else {
      nEvalIntervals = dmxvalue[0] - 192;
    }
    
    if (nEvalIntervals < 2) {
      nEvalIntervals = 20;
    } else if (nEvalIntervals > 61) {
      nEvalIntervals = 1;
    } else {
      nEvalIntervals = 20 - (nEvalIntervals / 3);
    }
  } else { // UI_MODE_PROG_2
    // Calculate the rate based on the potentiometer value
    // Dividing the pot value by 51 results in a range of approximately 1-21 eval
    // intervals per change
    nEvalIntervals = 21 - (CalcAvgPotVal() / 51);
  }
  return (nEvalIntervals);
}


void EvalOpProg3() {
  // Sequential Fade: Red->Yellow->Green->Cyan->Blue->Magenta->Red
  // 
  if (--ProgDelay == 0) {
    CalcIndexedRGB(ProgIndex);
    if (++ProgIndex == 1024) ProgIndex = 256;
    SetPWM(CurRed, CurGreen, CurBlue);
    ProgDelay = GetFadeDelay();
  }
}


void EvalUiProg3() {
  // Look for button presses (we ignore long button presses in this mode)
  if (ButtonPressShort) {
    // Move to program 4
    UIMode = UI_MODE_PROG_4;
    SetupProg4();
    SetupProg4Display();
  }
}


void SetupProg4() {
  OPMode = OP_MODE_PROG_4;
  SelectRandomColor();           // Setup 1st randomly selected color
  SetupInterpolator();
  ProgDelay = GetFadeDelay();  
}


// SelectRandomColor selects biased colors to ensure higher saturation.
// It does that by picking one intense color, a second less intense color
// and a third even less intense color then randomly assigning them to
// R, G and B.
void SelectRandomColor() {
  unsigned long NewColor[3];
  unsigned long ColorOrder;
  
  // Select the colors
  NewColor[0] = random(0xA0, 0x100);
  NewColor[1] = random(0x20, 0xB0);
  NewColor[2] = random(0x00, 0x18);
  
  // Select the assignment
  ColorOrder = random(0,6);
  switch (ColorOrder) {
    case 0: // RGB
      NextRed = NewColor[0];
      NextGreen = NewColor[1];
      NextBlue = NewColor[2];
      break;
      
    case 1: // RBG
      NextRed = NewColor[0];
      NextGreen = NewColor[2];
      NextBlue = NewColor[1];
      break;

    case 2: // GRB
      NextRed = NewColor[1];
      NextGreen = NewColor[0];
      NextBlue = NewColor[2];
      break;

    case 3: // GBR
      NextRed = NewColor[2];
      NextGreen = NewColor[0];
      NextBlue = NewColor[1];
      break;

    case 4: // BRG
      NextRed = NewColor[1];
      NextGreen = NewColor[2];
      NextBlue = NewColor[0];
      break;

    case 5: // BGR
      NextRed = NewColor[2];
      NextGreen = NewColor[1];
      NextBlue = NewColor[0];
      break;
  }
}


void SetupInterpolator() {
  int Temp;  // For the abs function (we're not supposed to include function calls in it)
  
  // Compute the delta and sign values
  Temp = (int) NextRed - (int) CurRed;
  DeltaVal[0] = abs(Temp);
  InterpSign[0] = (NextRed < CurRed);
  
  Temp = (int) NextGreen - (int) CurGreen;
  DeltaVal[1] = abs(Temp);
  InterpSign[1] = (NextGreen < CurGreen);
  
  Temp = (int) NextBlue - (int) CurBlue;
  DeltaVal[2] = abs(Temp);
  InterpSign[2] = (NextBlue < CurBlue);
  
  // Determine the major axis (will change by 1 per evaluation)
  // and the increment values for each axis (the 2 minor axis will be fractional)
  if ((DeltaVal[0] > DeltaVal[1]) && (DeltaVal[0] > DeltaVal[2])) {
    // Red is major
    IncVal[0] = 0x010000;
    IncVal[1] = (DeltaVal[1] << 16) / DeltaVal[0];
    IncVal[2] = (DeltaVal[2] << 16) / DeltaVal[0];
    Major = 0;
  } else if ((DeltaVal[1] > DeltaVal[0]) && (DeltaVal[1] > DeltaVal[2])) {
    // Green is major
    IncVal[0] = (DeltaVal[0] << 16) / DeltaVal[1];
    IncVal[1] = 0x010000;
    IncVal[2] = (DeltaVal[2] << 16) / DeltaVal[1];
    Major = 1;
  } else {
    // Blue is major
    IncVal[0] = (DeltaVal[0] << 16) / DeltaVal[2];
    IncVal[1] = (DeltaVal[1] << 16) / DeltaVal[2];
    IncVal[2] = 0x010000;
    Major = 2;
  }
  
  // Setup our interpolator color values (format: {Color[7:0] "." 0x0000 }
  InterpColor[0] = (unsigned long) CurRed << 16;
  InterpColor[1] = (unsigned long) CurGreen << 16;
  InterpColor[2] = (unsigned long) CurBlue << 16;
}


byte RoundFrac(long FracNum) {
  byte TempByte = (FracNum >> 16) & 0xFF;
  
  if ((FracNum & 0xFFFF) >= 0x8000) {
    // Round up
    return (TempByte + 1);
  } else {
    // No rounding
    return (TempByte);
  }
}


void EvalOpProg4() {
  // Random Fade: From one CurRGB to NextRGB
  // 
  if (--ProgDelay == 0) {
    if (DeltaVal[Major] != 0) {
      // Evaluate this step
      DeltaVal[Major]--;
      
      // Calculate the new Cur values
      for (byte i=0; i<3; i++) {
        if (InterpSign[i]) {
          // Decreasing color value
          InterpColor[i] -= IncVal[i];
        } else {
          // Increasing color value
          InterpColor[i] += IncVal[i];
        }
      }
    
      // Update the color
      CurRed = RoundFrac(InterpColor[0]);
      CurGreen = RoundFrac(InterpColor[1]);
      CurBlue = RoundFrac(InterpColor[2]);
      SetPWM(CurRed, CurGreen, CurBlue);
    } else {
      // Setup for the next fade
      SelectRandomColor();
      SetupInterpolator();
    }
    ProgDelay = GetFadeDelay();
  }
}


void EvalUiProg4() {
  // Look for button presses (we ignore long button presses in this mode)
  if (ButtonPressShort) {
    // Move to DMX mode
    UIMode = UI_MODE_DMX;
    SetupDMX();
    SetupDmxDisplay();
  }
}
#line 1 "storage.pde"
void CheckEEPsanity() {
  byte MagicByte = EEPROM.read(EEP_MAGIC_ADDR);
  
  if (MagicByte != EEP_MAGIC_BYTE) {
    // Set up the initial EEPROM values
    EEPROM.write(EEP_MAGIC_ADDR, EEP_MAGIC_BYTE);
    
    // Setup the initial DMX address to 001
    EEPROM.write(EEP_DMX_H_ADDR, 0);
    EEPROM.write(EEP_DMX_L_ADDR, 1);
    
    // Setup the initial operating mode to DMX
    EEPROM.write(EEP_USER_MODE, UI_MODE_DMX);
    
    // Setup the initial color to 0 (black)
    EEPROM.write(EEP_C_H_ADDR, 0);
    EEPROM.write(EEP_C_L_ADDR, 0);
  }
}


unsigned int GetDmxAddress() {
  byte HighByte = EEPROM.read(EEP_DMX_H_ADDR);
  byte LowByte = EEPROM.read(EEP_DMX_L_ADDR);
  unsigned int Addr = (HighByte << 8) | LowByte;
  
  if (Addr == 0) {
    Addr = 1;
  } else if (Addr > MAX_DMX_ADDR) {
    Addr = MAX_DMX_ADDR;
  }
  
  return (Addr);
}


void StoreDmxAddress(unsigned int workingAddr) {
  unsigned int TrueAddress = workingAddr - 3;  // Adjust for the real address
  
  if (GetDmxAddress() != TrueAddress) {
    EEPROM.write(EEP_DMX_H_ADDR, (TrueAddress >> 8) & 0x01);
    EEPROM.write(EEP_DMX_L_ADDR, (TrueAddress & 0xFF));
  }
}


byte GetFixtureMode() {
  byte EepData = EEPROM.read(EEP_USER_MODE);
  
  if ((EepData != UI_MODE_DMX) && (EepData != UI_MODE_PROG_1) &&
      (EepData != UI_MODE_PROG_2) && (EepData != UI_MODE_PROG_3) &&
      (EepData != UI_MODE_PROG_4)) {
    // Illegal value
    EepData = UI_MODE_DMX;
  }
  
  return (EepData);
}


void StoreFixtureMode(byte nMode) {
  if (GetFixtureMode() != nMode) {
    EEPROM.write(EEP_USER_MODE, nMode);
  }
}


unsigned int GetFixtureIndexedColor() {
  byte HighByte = EEPROM.read(EEP_C_H_ADDR);
  byte LowByte = EEPROM.read(EEP_C_L_ADDR);
  unsigned int fColor = (HighByte << 8) | LowByte;
  
  return(fColor & 0x3FF);  // limit to 0-1023
}


void StoreFixtureIndexedColor(unsigned int cIndex) {
  byte HighByte = (cIndex >> 8) & 0x03;
  byte LowByte = cIndex & 0xFF;
  
  if (GetFixtureIndexedColor() != cIndex) {
    EEPROM.write(EEP_C_H_ADDR, HighByte);
    EEPROM.write(EEP_C_L_ADDR, LowByte);
  }
}
  

// Function to only update mode-related information when we've remained in a mode
// for longer than a minimum threshold (to reduce EEPROM writes when moving
// quickly through modes)
void EvalModeChange() {
  
  // Look to see if we need to start/re-start the timer
  if ((UIMode != PrevUIMode) && (UIMode != UI_MODE_SET_ADDR)) {
    // Start the timer
    ModeChangeTimer = MODE_CHNG_CNT + 1;
    
    PrevUIMode = UIMode;
  }
  
  // Evalute the timer to see if it's time to update
  if (ModeChangeTimer != 0) {
    if (--ModeChangeTimer == 0) {
      // Store the new mode
      StoreFixtureMode(UIMode);
      
      // Check if we need to store the current indexed color
      if (UIMode == UI_MODE_PROG_2) {
        // Store the color if we're in strobe mode
        StoreFixtureIndexedColor(PrevPotVal);
      }
    }
  }
}

